# -*- coding: utf-8 -*-
###########################################################################
#     Sint Wind PI
#     Copyright 2012 by Tonino Tarsi <tony.tarsi@gmail.com>
#   
#     Please refer to the LICENSE file for conditions 
#     Visit http://www.vololiberomontecucco.it
# 
##########################################################################

"""WhatsApp plugin."""

import threading
import sys

from TTLib import  *

from yowsup.layers.protocol_media.protocolentities import *
from yowsup.layers.protocol_media.mediauploader import MediaUploader
from yowsup.layers.interface                           import YowInterfaceLayer, ProtocolEntityCallback
from yowsup.layers.protocol_messages.protocolentities  import TextMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities  import ImageDownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_receipts.protocolentities  import OutgoingReceiptProtocolEntity
from yowsup.layers.protocol_media.protocolentities  import LocationMediaMessageProtocolEntity
from yowsup.layers.protocol_acks.protocolentities      import OutgoingAckProtocolEntity
from yowsup.layers.protocol_media.protocolentities  import VCardMediaMessageProtocolEntity
from yowsup.layers import YowLayerEvent
from yowsup.layers.auth                        import YowCryptLayer, YowAuthenticationProtocolLayer, AuthError
from yowsup.layers.coder                       import YowCoderLayer
from yowsup.layers.network                     import YowNetworkLayer
from yowsup.layers.protocol_messages           import YowMessagesProtocolLayer
from yowsup.layers.protocol_media              import YowMediaProtocolLayer
from yowsup.layers.stanzaregulator             import YowStanzaRegulator
from yowsup.layers.protocol_receipts           import YowReceiptProtocolLayer
from yowsup.layers.protocol_acks               import YowAckProtocolLayer
from yowsup.layers.axolotl                     import YowAxolotlLayer
from yowsup.layers.protocol_iq                 import YowIqProtocolLayer
from yowsup.layers.protocol_calls              import YowCallsProtocolLayer

from yowsup.common import YowConstants
from yowsup.stacks import YowStack, YOWSUP_CORE_LAYERS
from yowsup import env

import traceback
import urllib


class swpi_plugin(threading.Thread):  #  do not change the name of the class
    
    def __init__(self, cfg):
        self.cfg = cfg
        self.last_timestamp = 0;
        threading.Thread.__init__(self)
        
        ###################### Plugin Initialization ################
        log("Intitializing plug-in %s" % sys.modules[__name__])
        
        self.phone = "0000000"
        self.password = "0000000000000000000="
        

        ###################### End Initialization ##################
        
    def run(self):
        log("Starting plugin : %s" % sys.modules[__name__])
   
        CREDENTIALS = (self.phone, self.password)  # replace with your phone and password       
     
        while (1) :
            try:
                log("WhatsApp-Plugin : Connecting to WhatsApp")
                layers = (
                     EchoLayer,
                    (YowAuthenticationProtocolLayer, YowMessagesProtocolLayer, YowReceiptProtocolLayer, 
                     YowAckProtocolLayer,YowMediaProtocolLayer, YowIqProtocolLayer, YowCallsProtocolLayer),
                    YowAxolotlLayer,
                    ) + YOWSUP_CORE_LAYERS
                stack = YowStack(layers)
                stack.setProp(YowAuthenticationProtocolLayer.PROP_CREDENTIALS, CREDENTIALS)  # setting credentials
                stack.setProp(YowNetworkLayer.PROP_ENDPOINT, YowConstants.ENDPOINTS[0])  # whatsapp server address
                stack.setProp(YowCoderLayer.PROP_DOMAIN, YowConstants.DOMAIN)              
                #stack.setProp(YowCoderLayer.PROP_RESOURCE, env.CURRENT_ENV.getResource())  # info about us as WhatsApp client
                stack.setProp(YowCoderLayer.PROP_RESOURCE, env.YowsupEnv.getCurrent().getResource())
            
                stack.broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_CONNECT))  # sending the connect signal
                stack.loop(timeout=2.5, discrete=0.5)
                time.sleep(10)
            except AuthError as e:
                log("WhatsApp-Plugin :  AuthError")
                print e.message
                print e.__class__.__name__
                traceback.print_exc(e) 
                time.sleep(60)
            
            except Exception as e:
                log("WhatsApp-Plugin :  Other Error")
                print e.message
                print e.__class__.__name__
                traceback.print_exc(e) 
                time.sleep(60)            


class EchoLayer(YowInterfaceLayer):

    def onRequestUploadResult(self, jid, filePath, resultRequestUploadIqProtocolEntity, requestUploadIqProtocolEntity, caption=None):

        if requestUploadIqProtocolEntity.mediaType == RequestUploadIqProtocolEntity.MEDIA_TYPE_AUDIO:
            doSendFn = self.doSendAudio
        else:
            doSendFn = self.doSendImage

        if resultRequestUploadIqProtocolEntity.isDuplicate():
            doSendFn(filePath, resultRequestUploadIqProtocolEntity.getUrl(), jid,
                             resultRequestUploadIqProtocolEntity.getIp(), caption)
        else:
            successFn = lambda filePath, jid, url: doSendFn(filePath, url, jid, resultRequestUploadIqProtocolEntity.getIp(), caption)
            mediaUploader = MediaUploader(jid, self.getOwnJid(), filePath,
                                      resultRequestUploadIqProtocolEntity.getUrl(),
                                      resultRequestUploadIqProtocolEntity.getResumeOffset(),
                                      successFn, self.onUploadError, self.onUploadProgress, async=False)
            mediaUploader.start()

    def doSendImage(self, filePath, url, to, ip=None, caption=None):
        entity = ImageDownloadableMediaMessageProtocolEntity.fromFilePath(filePath, url, ip, to, caption=caption)
        self.toLower(entity)

    def onUploadError(self, filePath, jid, url):
        print ("Upload file %s to %s for %s failed!" % (filePath, url, jid))

    def onUploadProgress(self, filePath, jid, url, progress):
        sys.stdout.write("%s => %s, %d%% \r" % (os.path.basename(filePath), jid, progress))
        sys.stdout.flush()


    def doSendAudio(self, filePath, url, to, ip=None, caption=None):
        entity = AudioDownloadableMediaMessageProtocolEntity.fromFilePath(filePath, url, ip, to)
        self.toLower(entity)


    @ProtocolEntityCallback("message")
    def onMessage(self, messageProtocolEntity):

        try:
            #print messageProtocolEntity
            Messaggio = ""
            textmsg = ""
            hexmsg = ""
            
    
            
            # Process message    
            if messageProtocolEntity.getType() == 'text':
                textmsg = messageProtocolEntity.getBody()
                hexmsg = ":".join("{:02x}".format(ord(c)) for c in textmsg)
                #log("WhatsApp-Plugin :  DEBUG " +  messageProtocolEntity.getFrom() + " hexmsg: " + hexmsg)               
    
            if (textmsg.lower().startswith("ciao jessica")) :
                
                Messaggio = "Ciao caro " + b"\xF0\x9F\x92\x8B" + " .\n"
                Messaggio += "Mi sento molto sola oggi. \n"
                Messaggio += "Se vuoi i miei dati fammi vedere come batte il tuo cuoricino " + b"\xE2\x9D\xA4" + " per me.\n"
                Messaggio += "Se vuoi vedere attraverso i miei occhi mandami una bella " + b"\xF0\x9F\x93\xB7" 
    
            # CAMERA
            if hexmsg.startswith("f0:9f:93:b7"):
                try:
                    log("WhatsApp-Plugin :  camera message from " + messageProtocolEntity.getFrom())               
                    f = urllib.urlopen("http://www.vololiberomontecucco.it/webcam/swpi_lastcamera1file.php")
                    jpg = f.read().split("\n")[0]
                    f.close()
                    txt_date = jpg.split("_")[1][:-5]
                    Messaggio = b"\xF0\x9F\x93\xB7" + " di " +  standbyCamera(txt_date)+ "\n"
                    Messaggio += jpg
                except:
                    pass
                
    
    
            #Just ack if message is old
            timeOfMessage = messageProtocolEntity.getTimestamp()
            now = time.time()
            sec = (now - timeOfMessage)
            if (sec > 300) : 
                self.toLower(messageProtocolEntity.ack())
                self.toLower(messageProtocolEntity.ack(True))
                return
            
            if ( now - self.last_timestamp < 30  ):
                time.sleep(30)
            self.last_timestamp = now;
            
            #if hexmsg == "e2:9d:a4" and globalvars.meteo_data.last_measure_time == None :
            if hexmsg.startswith("e2:9d:a4") and globalvars.meteo_data.last_measure_time == None :
                Messaggio = "Nessun dato ancora"
    
            # if ( textmsg == u'2764' ) 
            #if hexmsg == "e2:9d:a4" and globalvars.meteo_data.last_measure_time != None  :
            if hexmsg.startswith("e2:9d:a4") and globalvars.meteo_data.last_measure_time != None  :
                log("WhatsApp-Plugin :  meteo message from " + messageProtocolEntity.getFrom())               
    
                mydata = {} 
                mydata['last_measure_time'] = (globalvars.meteo_data.last_measure_time.strftime("[%d/%m/%Y-%H:%M:%S]"))
                mydata['idx'] = (globalvars.meteo_data.idx.strftime("[%d/%m/%Y-%H:%M:%S]"))
                mydata['wind_dir_code'] = (globalvars.meteo_data.wind_dir_code)
                mydata['wind_dir'] = (globalvars.meteo_data.wind_dir)
                mydata['wind_ave'] = (globalvars.meteo_data.wind_ave)
                mydata['wind_gust'] = (globalvars.meteo_data.wind_gust)
                mydata['temp_out'] = (globalvars.meteo_data.temp_out)
                mydata['abs_pressure'] = (globalvars.meteo_data.abs_pressure)
                mydata['rel_pressure'] = (globalvars.meteo_data.rel_pressure)
                mydata['hum_out'] = (globalvars.meteo_data.hum_out)
                mydata['rain'] = (globalvars.meteo_data.rain)
                mydata['rain_rate'] = (globalvars.meteo_data.rain_rate)
                mydata['temp_in'] = (globalvars.meteo_data.temp_in)
                mydata['hum_in'] = (globalvars.meteo_data.hum_in)
                mydata['wind_chill'] = (globalvars.meteo_data.wind_chill)
                mydata['temp_apparent'] = (globalvars.meteo_data.temp_apparent)
                mydata['dew_point'] = (globalvars.meteo_data.dew_point)
                mydata['cloud_base_altitude'] = (globalvars.meteo_data.cloud_base_altitude)
                mydata['uv'] = (globalvars.meteo_data.uv)
                mydata['illuminance'] = (globalvars.meteo_data.illuminance)
                mydata['winDayMin'] = (globalvars.meteo_data.winDayMin)
                mydata['winDayMax'] = (globalvars.meteo_data.winDayMax)
                mydata['winDayGustMin'] = (globalvars.meteo_data.winDayGustMin)
                mydata['winDayGustMax'] = (globalvars.meteo_data.winDayGustMax)
                mydata['TempOutMin'] = (globalvars.meteo_data.TempOutMin)
                mydata['TempOutMax'] = (globalvars.meteo_data.TempOutMax)
                mydata['TempInMin'] = (globalvars.meteo_data.TempInMin)
                mydata['TempInMax'] = (globalvars.meteo_data.TempInMax)
                mydata['UmOutMin'] = (globalvars.meteo_data.UmOutMin)
                mydata['UmOutMax'] = (globalvars.meteo_data.UmOutMax)
                mydata['UmInMin'] = (globalvars.meteo_data.UmInMin)
                mydata['UmInMax'] = (globalvars.meteo_data.UmInMax)
                mydata['PressureMin'] = (globalvars.meteo_data.PressureMin)
                mydata['PressureMax'] = (globalvars.meteo_data.PressureMax)
                mydata['wind_dir_ave'] = (globalvars.meteo_data.wind_dir_ave)
                mydata['rain_rate_24h'] = (globalvars.meteo_data.rain_rate_24h)
                mydata['rain_rate_1h'] = (globalvars.meteo_data.rain_rate_1h)
                        
                #Messaggio = "     " + messageProtocolEntity.getBody()
                Messaggio = ""
                Messaggio += "Ciao tesoro " +  b"\xF0\x9F\x92\x8B" + "\n"
                if (mydata["last_measure_time"] != None) :
                    Messaggio += b"\xF0\x9F\x95\x9D" +  "Dati di " + standby(mydata["last_measure_time"]) + "\n"
                if (mydata["wind_dir_code"] != None) :
                    Messaggio +=  b"\xF0\x9F\x94\x84" + "Direzione    : " + mydata["wind_dir_code"] + "\n"
                if ((mydata["wind_ave"] != None)  and   (mydata["wind_gust"] != None))  :
                    Messaggio += b"\xF0\x9F\x92\xA8" +  "Vel. media  : " + str(int(mydata["wind_ave"])) + " kmh\n"
                    Messaggio += b"\xF0\x9F\x92\xA8" +  "Raffica        : " + str(int(mydata["wind_gust"])) + " kmh\n"
                if (mydata["temp_out"] != None):
                    Messaggio += b"\xE2\x99\xA8" +       "Temp.         : " + str(int(mydata["temp_out"])) + " gradi\n"
                if (mydata["abs_pressure"] != None):
                    Messaggio += b"\xF0\x9F\x8F\xA7" +   "Pressione   : " + str(int(mydata["rel_pressure"])) + " hpa\n"    
                if (mydata["hum_out"] != None):
                    Messaggio += b"\xF0\x9F\x92\xA7" +   "Umidità     : " + str(int(mydata["hum_out"])) + " %\n"                    
                if (mydata["cloud_base_altitude"] != None):
                    Messaggio += b"\xE2\x98\x81" +       "Base cumulo: " + str(int(mydata["cloud_base_altitude"])) + " m\n"                 
    
            if (Messaggio != ""):
                self.toLower(TextMessageProtocolEntity(Messaggio, to=messageProtocolEntity.getFrom()))
            self.toLower(messageProtocolEntity.ack())
            self.toLower(messageProtocolEntity.ack(True))

        except Exception as e:
            log("WhatsApp-Plugin :  Message Error")
            print e.message
            print e.__class__.__name__
            traceback.print_exc(e) 
            self.toLower(messageProtocolEntity.ack())
            self.toLower(messageProtocolEntity.ack(True))  

    @ProtocolEntityCallback("receipt")
    def onReceipt(self, entity):
        ack = OutgoingAckProtocolEntity(entity.getId(), "receipt", entity.getType(), entity.getFrom())
        self.toLower(entity.ack())
        
    @ProtocolEntityCallback("event")
    def onEvent(self, layerEvent):
        log("WhatsApp-Plugin : EVENT " + layerEvent.getName())
        if layerEvent.getName() == YowNetworkLayer.EVENT_STATE_DISCONNECTED:
            msg = "WhatsApp-Plugin : Disconnected reason: %s" % layerEvent.getArg("reason")
            #SendMail(self.cfg, "WhatsApp-Plugin : Disconnected",msg, "")
            log(msg)
            if layerEvent.getArg("reason") == 'Connection Closed':
                time.sleep(20)
                log("WhatsApp-Plugin : Issueing EVENT_STATE_CONNECT")
                self.getStack().broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_CONNECT))
            elif layerEvent.getArg("reason") == 'Ping Timeout':
                time.sleep(20)
                log("WhatsApp-Plugin : Issueing EVENT_STATE_DISCONNECT")
                self.getStack().broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_DISCONNECT)) 
                time.sleep(20)
                log("WhatsApp-Plugin : Issueing EVENT_STATE_CONNECT")
                self.getStack().broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_CONNECT))
        elif layerEvent.getName() == YowNetworkLayer.EVENT_STATE_CONNECTED:
            log("WhatsApp-Plugin : Connected")
        elif layerEvent.getName() == YowNetworkLayer.EVENT_STATE_DISCONNECT:
            log("WhatsApp-Plugin : disconnect")
            


def standbyCamera(valore):

    d1 = datetime.datetime.strptime(valore, "%d%m%Y-%H%M%S")
    d2 = datetime.datetime.strptime(datetime.datetime.today().strftime("%d-%m-%Y %H:%M:%S"), "%d-%m-%Y %H:%M:%S")
    sec = (d2 - d1).seconds
    if (sec < 60) :
        return ("%d secondi fà" % sec) #+ b"\xA0"
    elif ( sec < 3600 ):
        return ( "%d minuti fà" % int(round(sec / 60)) ) # + b"\xA0"
    else:
        return ( "%d ore fà" % int(round(sec / 3600)) ) # + b"\xA0"


def standby(valore):
    valore = valore.replace("[", "")
    valore = valore.replace("]", "")
    valore = valore.replace("-", " ")
    valore = valore.replace("/", "-")

    d1 = datetime.datetime.strptime(valore, "%d-%m-%Y %H:%M:%S")
    d2 = datetime.datetime.strptime(datetime.datetime.today().strftime("%d-%m-%Y %H:%M:%S"), "%d-%m-%Y %H:%M:%S")
    sec = (d2 - d1).seconds
    if (sec < 60) :
        return ("%d secondi fà" % sec) #+ b"\xA0"
    else:
        return ( "%d minuti fà" % int(round(sec / 60, -1)) ) # + b"\xA0"


if __name__ == '__main__':

    print "starting"
    configfile = '../swpi.cfg'
    cfg = config.config(configfile)
    plg =  swpi_plugin(cfg)
    plg.start()
